@@ -10,23 +10,22 @@
 #include "matrix.h"
 
 void free_mat(matrix *M){
-
   Free(M->entries);
   Free(M);
+}
 
+void free_mat3(matrix3 *M){
+  Free(M->entries);
+  Free(M);
 }
 
 void free_vec(vector *V){
-
   Free(V->entries);
   Free(V);
-
 }
 
 int nrow_matrix(matrix *M){
-
   return M->nr;
-
 }
 
 int ncol_matrix(matrix *M){
@@ -279,6 +278,90 @@ void MtM(matrix *M, matrix *A){
   }
 }
 
+// Does cholesky of := A, where A is symmetric positive definite, of order *n
+void cholesky(matrix *A, matrix *AI){ // {{{ 
+
+  if( !(nrow_matrix(A)  == ncol_matrix(A) && 
+	nrow_matrix(AI) == ncol_matrix(AI) &&
+	nrow_matrix(A)  == ncol_matrix(AI)) ){
+    oops("Error: dimensions in invertSPD\n");
+  }
+
+  // Ensure that A and AI do not occupy the same memory. 
+  if(A != AI){
+//	  printf(" er her\n"); 
+    choleskyunsafe(A, AI);
+  } else {
+    // if M and A occupy the same memory, store the results in a
+    // temporary matrix. 
+    matrix *temp;
+    malloc_mat(nrow_matrix(AI),ncol_matrix(AI),temp);
+
+    choleskyunsafe(A, temp);
+    
+    // Copy these results into AI, then remove the temporary matrix
+    mat_copy(temp,AI);    
+    free_mat(temp);
+  }
+
+} // }}} 
+
+// cholesky := A, where A is symmetric positive definite, of order *n
+void choleskyunsafe(matrix *A, matrix *AI){ // {{{ 
+  //unsafe because it assumes A and AI are both square and of the same
+  //dimensions, and that they occupy different memory
+
+  char uplo = 'U'; // lower version 
+  int i, j;
+  int n = nrow_matrix(A);
+  int lda = n; // matrix A has dimensions *n x *n
+  int info = -999;
+  double rcond;
+  int pivot[n];
+  double z[n];
+  double qraux[n];
+  double work[2*n];
+  int rank = 0;
+  int job=1;
+  double tol = 1.0e-07;
+  
+// First copy the matrix A into the matrix AI
+//  print_mat(A); 
+   mat_copy(A,AI); 
+//  print_mat(AI); 
+
+//  printf("sssssssssss======================\n"); 
+  job = 1; // Indicates that AI is upper triangular
+  rcond = 999.0;
+
+    // First find the Cholesky factorization of A,
+    // stored as an upper triangular matrix
+    char uplo1 = 'U'; // lower version 
+    F77_CALL(dpotrf)(&uplo1, &n, AI->entries, &n, &info); 
+
+    // Lastly turn the vector a into the matrix AI
+    // Take only the lower triangular portion, since this 
+    // is the relevant part returned by dpotrf
+    for(i = 0; i < n; i++){
+      for(j = 0; j < i; j++){
+	      ME(AI,i,j) = 0; 
+      }
+    }
+//    print_mat(AI); 
+
+//    Rprintf("in chol \n"); 
+//    printf("======================\n"); 
+//    print_mat(A); 
+//    print_mat(AI); 
+//    printf(" check chol back\n"); 
+//    matrix *tmp; 
+//    malloc_mat(n,n,tmp); 
+//    MtM(AI,tmp); 
+//    print_mat(tmp); 
+//    free_mat(tmp); 
+
+} // }}} 
+
 // Does AI := inverse(A), where A is symmetric positive definite, of order *n
 void invertSPD(matrix *A, matrix *AI){
 
@@ -389,6 +472,7 @@ void invertSPDunsafe(matrix *A, matrix *AI){
 	ME(AI,i,j) = ME(AI,j,i);
       }
     }
+    
   }
 
 }
@@ -1109,8 +1193,9 @@ void invertUnsafe(matrix *A, matrix *Ainv){
 
   if(info != 0){
     //Avoid printing this error message
-    Rprintf("Error in invert: DGETRF returned info = %d \n",info);
+    Rprintf("2 Error in invert: DGETRF returned info = %d \n",info);
     mat_zeros(Ainv);
+    print_mat(Ainv); 
   } else {
   
     for(i = 0; i < n; i++){
@@ -1120,7 +1205,7 @@ void invertUnsafe(matrix *A, matrix *Ainv){
     
     if(info != 0){
       //Avoid printing this error message
-      Rprintf("Error in invert: DGETRF returned info = %d \n",info);
+      Rprintf("1 Error in invert: DGETRF returned info = %d \n",info);
       mat_zeros(Ainv);
       return;
     } 
@@ -1189,7 +1274,7 @@ void invertUnsafeS(matrix *A, matrix *Ainv,int silent){
   if(info != 0){
     //Avoid printing this error message
     mat_zeros(Ainv);
-    if (silent==0) Rprintf("Error in invert: DGETRF returned info = %d \n",info);
+    if (silent==0) Rprintf("3 Error in invert: DGETRF returned info = %d \n",info);
   } else {
   
     for(i = 0; i < n; i++){
@@ -1201,7 +1286,7 @@ void invertUnsafeS(matrix *A, matrix *Ainv,int silent){
       //Avoid printing this error message
       mat_zeros(Ainv);
       free(work); free(iwork); free(dwork); free(ipiv);
-      if (silent==0) Rprintf("Error in invert: DGETRF returned info = %d \n",info);
+      if (silent==0) Rprintf("4 Error in invert: DGETRF returned info = %d \n",info);
       return;
     } 
     
